package me.icymint.libra.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import org.dom4j.Document;
import org.dom4j.DocumentFactory;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

/**
 * 提供解析和保存XML文档的模板代码。
 * 
 * @author Daniel Yu
 * @date 2013-3-12
 * 
 * @param <T>
 * @param <V>
 */
public abstract class AbstractParser<T, V> implements XmlParser<T, V> {

	/**
	 * 创建头部信息，其同时负责创建根元素。
	 * 
	 * @throws XmlParserException
	 */
	protected abstract Element createHeader() throws XmlParserException;

	/**
	 * 过滤Document，为其添加DocType等信息。
	 * 
	 * @param doc
	 */
	protected Document filter(Document doc) {
		return doc;
	}

	/**
	 * 输出格式的设定。
	 * 
	 * @param of
	 */
	protected void filter(OutputFormat of) {
		of.setEncoding("UTF-8");
		of.setTrimText(false);
		of.setIndent("	");
	}

	/**
	 * 具体的解析根元素。
	 * 
	 * @param root
	 * @param properties
	 *            公共属性。
	 * @throws XmlParserException
	 */
	protected abstract T parser(Element root, V properties)
			throws XmlParserException;

	@Override
	public T parser(InputStream is, V properties) throws XmlParserException {
		Element root = validate0(is);
		return root == null ? null : parser(root, properties);
	}

	@Override
	public void store(T t, OutputStream os) throws XmlParserException {
		Element root = createHeader();
		transfer(root, t);
		XMLWriter x;
		try {
			OutputFormat of = OutputFormat.createPrettyPrint();
			filter(of);
			x = new XMLWriter(os, of);
			Document doc = DocumentFactory.getInstance().createDocument(root);
			x.write(filter(doc));
		} catch (UnsupportedEncodingException e1) {
			throw new XmlParserException(e1);
		} catch (IOException e1) {
			throw new XmlParserException(e1);
		}
	}

	/**
	 * 把对象转换成xml树形对象。
	 * 
	 * @param root
	 * @param t
	 * @throws XmlParserException
	 */
	protected abstract void transfer(Element root, T t)
			throws XmlParserException;

	@Override
	public boolean validate(InputStream is) throws XmlParserException {
		return this.validate0(is) != null;
	}

	/**
	 * 检验输入流，并解析成根元素。
	 * 
	 * @param is
	 * @throws XmlParserException
	 */
	protected abstract Element validate0(InputStream is)
			throws XmlParserException;

}
