package me.icymint.libra.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentFactory;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.SAXValidator;
import org.dom4j.io.XMLWriter;
import org.dom4j.util.XMLErrorHandler;
import org.xml.sax.SAXException;

/**
 * Schema类XML文档解析和保存，其需要提供xsd文档位置、根元素名称、命名空间以及xsd网络路径。
 * 
 * @author Daniel Yu
 * @date 2013-3-12
 * 
 * @param <T>
 * @param <V>
 */
public abstract class AbstractSchemaParser<T, V> extends AbstractParser<T, V> {

	private final URL xsd;
	private final String rootName;
	private final String nameSpace;
	private final String xsdLocation;

	protected AbstractSchemaParser(URL xsd, String rootName, String nameSpace,
			String xsdLocation) {
		this.xsd = xsd;
		this.rootName = rootName;
		this.nameSpace = nameSpace;
		this.xsdLocation = xsdLocation;
	}

	@Override
	protected Element createHeader() throws XmlParserException {
		DocumentFactory df = DocumentFactory.getInstance();
		Element root = df.createElement(rootName, nameSpace);
		Namespace ns = df.createNamespace("xsi",
				"http://www.w3.org/2001/XMLSchema-instance");
		root.addAttribute(df.createQName("schemaLocation", ns), nameSpace + " "
				+ xsdLocation);
		return root;
	}

	@Override
	protected Element validate0(InputStream xml) throws XmlParserException {
		XMLErrorHandler errorHandler = new XMLErrorHandler();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		factory.setNamespaceAware(true);
		try {
			SAXParser parser = factory.newSAXParser();
			SAXReader xmlReader = new SAXReader();
			xmlReader.setEncoding("UTF-8");
			Document xmlDocument = xmlReader.read(xml);
			parser.setProperty(
					"http://java.sun.com/xml/jaxp/properties/schemaLanguage",
					"http://www.w3.org/2001/XMLSchema");
			parser.setProperty(
					"http://java.sun.com/xml/jaxp/properties/schemaSource",
					xsd.openStream());
			SAXValidator validator = new SAXValidator(parser.getXMLReader());
			validator.setErrorHandler(errorHandler);
			validator.validate(xmlDocument);
			XMLWriter writer = new XMLWriter(OutputFormat.createPrettyPrint());

			if (errorHandler.getErrors().hasContent()) {
				writer.write(errorHandler.getErrors());
				return null;
			} else {
				return xmlDocument.getRootElement();
			}
		} catch (ParserConfigurationException e) {
			throw new XmlParserException(e);
		} catch (SAXException e) {
			throw new XmlParserException(e);
		} catch (DocumentException e) {
			throw new XmlParserException(e);
		} catch (UnsupportedEncodingException e) {
			throw new XmlParserException(e);
		} catch (IOException e) {
			throw new XmlParserException(e);
		}
	}
}
